// ========================================================================================
//	TDocument.cpp			 	Copyright (C) 2001-2002 Mike Lockwood. All rights reserved.
// ========================================================================================
/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "config.h"
#include "FWCommon.h"

#include "TDocument.h"
#include "TDocumentWindow.h"
#include "TApplication.h"
#include "TMenu.h"
#include "TCommonDialogs.h"
#include "TCommandID.h"


TDocument::TDocument(TFile* file)
	:	TWindowContext(gApplication),
		fModified(false)
{
	if (file)
	{
		fFile.Specify(*file);
		fTitle = file->GetFileName();
	}
	else
		gApplication->GetUntitledDocumentTitle(fTitle);

	gApplication->AddSubContext(this);
}


TDocument::~TDocument()
{
	gApplication->RemoveSubContext(this);
}


const TChar* TDocument::GetTitle() const
{
	return fTitle;
}


void TDocument::Open(TDocumentWindow* window)
{
	window->SetWindowContext(this);

	if (fFile.IsSpecified())
	{
		ReadFromFile(&fFile);
		SetTitle(fFile.GetFileName());
	}
}


void TDocument::Save()
{
	if (! fFile.IsSpecified())
	{
		TFile* file = TCommonDialogs::SaveFile(NULL, GetMainWindow());
			
		if (!file)
			return; // mgl should use exceptions?

		SetFile(file);
	}

	WriteToFile(&fFile);
	SetModified(false);
}


void TDocument::SaveAs(bool makeCopy)
{
	TFile* file = TCommonDialogs::SaveFile(NULL, GetMainWindow());
	
	if (file)
	{
		if (! makeCopy)
			SetFile(file);

		WriteToFile(&fFile);

		if (! makeCopy)
			SetModified(false);
	}
}


void TDocument::ReadFromFile(TFile* /*file*/)
{
}


void TDocument::WriteToFile(TFile* /*file*/)
{
}


void TDocument::AddWindow(TTopLevelWindow* window)
{
	TWindowContext::AddWindow(window);
	
	if (fTitle.GetLength() > 0 && window == GetMainWindow())
		window->SetTitle(fTitle);
}


void TDocument::SetFile(TFile* file)
{
	ASSERT(file);

	fFile = *file;

	if (file->IsSpecified())
	{
		SetTitle(file->GetFileName());
		
		TTopLevelWindow* window = GetMainWindow();
		if (window)
			window->SetTitle(fTitle);
	}
	
	TTopLevelWindow* window = GetMainWindow();
	if (window && window->GetTarget())
		window->GetTarget()->HandleCommand(this, this, kFilePathChangedCommandID);
	else
		HandleCommand(this, this, kFilePathChangedCommandID);
}


void TDocument::SetTitle(const TChar* title)
{
	fTitle = title;
	
	TTopLevelWindow* window = GetMainWindow();
	if (window)
		window->SetTitle(title);
}


bool TDocument::AllowClose()
{
	if (IsModified())
	{
		bool save;

		if (!TCommonDialogs::SaveChanges(fTitle, GetMainWindow(), save))
			return false;

		if (save)
		{	
	 		// save the file
	 		if (fFile.IsSpecified())
				Save();
			else
			{
				TFile* file = TCommonDialogs::SaveFile(NULL, GetMainWindow());
				
				if (file)
				{
					SetFile(file);
					Save();
				}
				else
					return false;
			}
		}
	}

	return true;
}


void TDocument::DoSetupMenu(TMenu* menu)
{
	if (IsModified() || !fFile.IsSpecified())
		menu->EnableCommand(kSaveCommandID);

	menu->EnableCommand(kSaveAsCommandID);
	menu->EnableCommand(kSaveCopyCommandID);

	TWindowContext::DoSetupMenu(menu);
}


bool TDocument::DoCommand(TCommandHandler* sender, TCommandHandler* receiver, TCommandID command)
{
	switch (command)
	{
		case kSaveCommandID:
			Save();
			return true;

		case kSaveAsCommandID:
			SaveAs(false);
			return true;

		case kSaveCopyCommandID:
			SaveAs(true);
			return true;

		default:
			return TWindowContext::DoCommand(sender, receiver, command);
	}
}
