// ========================================================================================
//	TIDEApplication.cpp		 	Copyright (C) 2001-2002 Mike Lockwood. All rights reserved.
// ========================================================================================
/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "config.h"
#include "IDECommon.h"

#include "TIDEApplication.h"
#include "TProjectCommands.h"
#include "TProjectDocument.h"
#include "TLogDocument.h"
#include "TTextDocument.h"
#include "fw/TButton.h"
#include "fw/TChildProcess.h"
#include "fw/TCommonDialogs.h"
#include "fw/TDialogWindow.h"
#include "fw/TMenu.h"
#include "fw/TStaticText.h"
#include "fw/TTopLevelWindow.h"
#include "fw/TPixmap.h"
#include "fw/TWindowPositioners.h"

#ifdef ENABLE_DEBUGGER
#include "debugger/TDebuggerCommands.h"
#include "debugger/TExecutable.h"
#include "debugger/TProcess.h"
#include "debugger/TSymDocument.h"
#include "debugger/TThread.h"
#include "debugger/TThreadContext.h"
#include "debugger/TUnixLocalTarget.h"
#include "debugger/TVariable.h"
#include "debugger/TVariableListView.h"
#include "fw/TColumnResizer.h"
#include "fw/TDocumentWindow.h"
#include "fw/TScroller.h"
#include "fw/TTypeSelectBehavior.h"
#endif // ENABLE_DEBUGGER

#include "fw/intl.h"

#include <unistd.h>
#include <ctype.h>

#include "Pixmaps/zoinks.xpm"

#ifdef ENABLE_DEBUGGER
TTarget* TIDEApplication::sLocalTarget = NULL;
#endif


TIDEApplication::TIDEApplication(int argc, char **argv)
	:	TApplication(argc, argv)
{
#ifdef ENABLE_DEBUGGER
	sLocalTarget = new TUnixLocalTarget;
	sLocalTarget->AddListener(dynamic_cast<TTargetListener *>(this));
#endif
}


TIDEApplication::~TIDEApplication()
{
}


TPixmap* TIDEApplication::GetIcon()
{
	return new TPixmap(zoinks_xpm);
}


void TIDEApplication::DoSetupMenu(TMenu* menu)
{
	menu->EnableCommand(kAboutCommandID);
	menu->EnableCommand(kNewProjectCommandID);
	menu->EnableCommand(kCloseAllTextWindowsCommandID);
	menu->EnableCommand(kShowManPageCommandID);

#ifdef ENABLE_DEBUGGER
	menu->EnableCommand(kClearAllBreakpointsCommandID);
#endif

	TApplication::DoSetupMenu(menu);
}


bool TIDEApplication::DoCommand(TCommandHandler* sender, TCommandHandler* receiver, TCommandID command)
{
	switch (command)
	{
		case kAboutCommandID:
			ShowAboutBox();
			return true;
			
		case kNewProjectCommandID:
			NewProject();
			return true;
			
		case kCloseAllTextWindowsCommandID:
			CloseAllTextWindows();
			return true;

		case kShowManPageCommandID:
			ShowManPage();
			return true;

#ifdef ENABLE_DEBUGGER
		case kClearAllBreakpointsCommandID:
			ClearAllBreakpoints();
			return true;
#endif
			
		default:
			return TApplication::DoCommand(sender, receiver, command);
	}
}


void TIDEApplication::ShowAboutBox()
{
	TRect	dialogBounds(0, 0, 400, 170);
	TDialogWindow* dialog = new TDialogWindow(dialogBounds, _("About Zoinks"), true, NULL);

	TString	copyright(_("Zoinks "));
	copyright += VERSION;
	copyright += _(" (c) 2001-2002 Mike Lockwood.");

	TStaticText* staticText = new TStaticText(dialog, TRect(20, 20, dialogBounds.GetWidth() - 20, 36), copyright, kTextAlignCenter);
	staticText->SetWindowPositioner(CenterHorizontal);	
	staticText = new TStaticText(dialog, TRect(20, 36, dialogBounds.GetWidth() - 20, 52), _("All Rights Reserved."), kTextAlignCenter);
	staticText->SetWindowPositioner(CenterHorizontal);	
	staticText = new TStaticText(dialog, TRect(20, 52, dialogBounds.GetWidth() - 20, 68), _("http://zoinks.mikelockwood.com"), kTextAlignCenter);
	staticText->SetWindowPositioner(CenterHorizontal);	
	staticText = new TStaticText(dialog, TRect(20, 90, dialogBounds.GetWidth() - 20, 106), _("Zoinks is released under the GNU General Public License."), kTextAlignCenter);
	staticText->SetWindowPositioner(CenterHorizontal);	
	staticText = new TStaticText(dialog, TRect(20, 106, dialogBounds.GetWidth() - 20, 122), _("See \"COPYING\" for details."), kTextAlignCenter);
	staticText->SetWindowPositioner(CenterHorizontal);	

	TCoord buttonWidth = 80;
	TRect buttonBounds;
	buttonBounds.left = (dialogBounds.GetWidth() - buttonWidth) / 2;
	buttonBounds.right = buttonBounds.left + buttonWidth;
	buttonBounds.bottom = dialogBounds.bottom - 20;
	buttonBounds.top = buttonBounds.bottom - 20;

	TButton* button = new TButton(dialog, buttonBounds, _("OK"), kOKCommandID, true);
	button->SetWindowPositioner(CenterHorizontal);
	button->SetDefault();
	
	dialog->Show(true);

	gApplication->ModalDialog(dialog);
	dialog->Close();
}


void TIDEApplication::NewProject()
{
	TDocument* document = TProjectDocument::CreateDocument(NULL);
	OpenDocument(document);
}


void TIDEApplication::CloseAllTextWindows()
{
	TListIterator<TWindowContext>	iter(fSubContexts);
	TWindowContext* context;
	while ((context = iter.Next()) != NULL)
	{
		TTextDocument* document = dynamic_cast<TTextDocument*>(context);
		
		if (document && document->AllowClose())
			document->Close();
	}
}


void TIDEApplication::ShowManPage()
{
	TString text;
	if (TCommonDialogs::TextEntryDialog(_("Show man page for:"), _("Show Man Page"), NULL, text))
	{
		TString	command("man -a ");
		command += text;
//		command += " | col -b";	// this seems to be broken on Vine Linux and possibly other systems

		TChildProcess* child = Execute(command, NULL, false);

		if (child)
		{
			TLogDocument* document = new TLogDocument(NULL, false);
			document->AddBehavior(new TProjectBehavior);
		
			TString title("man ");
			title += text;
			document->SetTitle(title);
			OpenDocument(document);

			while (1)
			{
				char buffer[1000];
					
				ssize_t count = read(child->GetStdout(), buffer, sizeof(buffer) - 1);
				if (count > 0)
				{
					// remove backspaces
					uint32 bsCount = 0;
					const char* src = buffer;
					char* dest = buffer;
					const char* end = buffer + count;
					
					while (src < end)
					{
						if (*src == '\b')
						{
							if (dest == buffer)
								bsCount++;
							else
								dest--;
							
							src++;		
						}
						else
							*dest++ = *src++;
					}
					
					if (bsCount > 0)
						document->Truncate(bsCount);
					
					document->AppendText(buffer, dest - buffer, false);
				}
				else
					break;
			}
			
			// see if we got anything
			const TChar* output = document->GetText();
			STextOffset length = document->GetTextLength();
			bool gotText = false;
			
			for (STextOffset i = 0; !gotText && i < length; i++)
			{
				if (isalnum(output[i]))
					gotText = true;
			}
			
			if (!gotText)
			{
				TString	message(_("No man page for "));
				message += text;
				document->ClearText();
				document->AppendText(message, message.GetLength(), false);
			}
	
			delete child;
		}
	}
}


TDocument* TIDEApplication::CreateDocument(TFile* file)
{
	TDocument* document;

#ifdef ENABLE_DEBUGGER	
	document = TSymDocument::CreateDocument(file);
	if (document)
		return document;
#endif

	if (file)
	{
		document = TProjectDocument::CreateDocument(file);
		if (document)
			return document;
	}

	return TApplication::CreateDocument(file);
}


#ifdef ENABLE_DEBUGGER
void TIDEApplication::ClearAllBreakpoints()
{
	TExecutable::ClearAllBreakpoints();
}


void TIDEApplication::MakeVariableWindow(TVariable* variable, TStackFrame* stackFrame)
{
	ASSERT(variable);

	TDocumentWindow* window = new TDocumentWindow(TRect(0, 0, 500, 500), variable->GetName());
	window->SetWindowContext(this);
	TRect bounds = window->GetBounds();

/*
	TMenuBar* menuBar = MakeMenuBar(window);
	menuBar->SetWindowPositioner(WidthParent);
	
	bounds.top = menuBar->GetBounds().bottom;
*/
	TScroller* scroller = new TScroller(window, bounds, true, true);
	scroller->SetWindowPositioner(SizeRelativeParent);

	TVariableListView* variableListView = new TVariableListView(scroller, TRect(0, 0, 200, 200), variable->GetProcess());
	scroller->SetContainedView(variableListView);
	variableListView->AddBehavior(new TTypeSelectBehavior);
	variableListView->AddBehavior(new TColumnResizer);
	window->SetTarget(variableListView);

	window->Show(true);
	
	// need to do this after the view is created
	TCoord width = (variableListView->GetWidth() - variableListView->GetColumnWidth(0)) / 2;
	variableListView->SetColumnWidth(1, width);
	variableListView->SetColumnWidth(2, width);
	
	variableListView->SetStackFrame(stackFrame);
}
#endif // ENABLE_DEBUGGER


TFile* TIDEApplication::FindFile(const char* fileName, TDocument* baseDocument, TDocument* baseDocument2)
{
	ASSERT(baseDocument2 == NULL);

	return TApplication::FindFile(fileName, baseDocument, TProjectDocument::GetCurrentProject());
}


#ifdef ENABLE_DEBUGGER

void TIDEApplication::ProcessCreated(TProcess* process)
{
	process->AddListener(dynamic_cast<TProcessListener *>(this));
}


void TIDEApplication::ThreadCreated(TThread* thread)
{
	thread->AddListener(dynamic_cast<TThreadListener *>(this));
}


void TIDEApplication::ThreadSuspended(TThread* thread)
{
	// make sure we have a thread context
	TListIterator<TWindowContext> iter(fSubContexts);

	TWindowContext* context;
	bool found = false;
	while ((context = iter.Next()) != NULL && !found)
	{
		TThreadContext* threadContext = dynamic_cast<TThreadContext*>(context);
		if (threadContext && threadContext->GetThread() == thread)
			found = true;
	}

	if (!found)
	{
		TThreadContext* context = new TThreadContext(thread);
		AddSubContext(context);

//		ThreadSuspended will happen automatically since the TThreadContext will be added to the end of the listener list during iteration.
//		context->ThreadSuspended(thread);
	}
}

void TIDEApplication::ChildDied(pid_t pid, int status)
{
	TChildProcess* child = fChildren.Search(&pid);

	if (child)
		child->Terminated(status);
	else
		((TUnixLocalTarget *)sLocalTarget)->ProcessSignal(pid, status);
}

#endif // ENABLE_DEBUGGER
